/* SISTEMAS INTEGRADOS TECNISA
   PLANEJAMENTO ALIMENTAO INDUSTRIAL

NOME: CUSTO_PRODUTO_LCEST.SQL

DESCRIO:
RETORNA O CUSTO DE UM PRODUTO EM UMA DETERMINADA DATA PARA UM DETERMINADO LOCAL DE ESTOQUE.

P_CDFILIAL     - CDIGO DA FILIAL.
P_CDPRODUTO    - CDIGO DO PRODUTO.
P_NRTIPOCUST   - NMERO DO TIPO DE CUSTO (1 A 10).
P_DTREFERENCIA - DATA DO CUSTO.
P_CUSTO        - RETORNA O CUSTO DO PRODUTO.

CRIAO:
20/11/2001 - RICARDO JULIANO

MODIFICADA PARA FUNO:
18/06/2004 - LEONARDO LEO
*/

IF EXISTS (SELECT *
           FROM SYSOBJECTS O, SYSUSERS U
           WHERE O.NAME = 'CUSTO_PRODUTO_LCEST' AND
                 TYPE = 'FN' AND
                 U.UID = O.UID  AND
                 U.NAME = USER)
DROP FUNCTION CUSTO_PRODUTO_LCEST
GO

CREATE FUNCTION CUSTO_PRODUTO_LCEST
   (@P_CDFILIAL     VARCHAR(04),
    @P_CDPRODUTO    VARCHAR(10),
    @P_NRTIPOCUSTO  NUMERIC,
    @P_DTREFERENCIA DATETIME,
    @P_DTATUAL      DATETIME,
    @P_ALMOXARIFE   VARCHAR(06) = ' ',
    @P_LOCALESTOQ   VARCHAR(06) = ' ',
    @P_NRLOTEESTQ   VARCHAR(20) = ' ',
    @P_NRSUBLOTE    VARCHAR(04) = ' ')

RETURNS DOUBLE PRECISION

AS

BEGIN

   DECLARE @V_VRCUSTO        DOUBLE PRECISION;
   DECLARE @V_SEM_IMPOSTO    DOUBLE PRECISION;
   DECLARE @V_DTREFERENCIA   DATETIME;

   DECLARE @V_QTESTOQDIA     DOUBLE PRECISION;
   DECLARE @V_QTESTOQANT     DOUBLE PRECISION;
   DECLARE @V_VRESTOQDIA     DOUBLE PRECISION;
   DECLARE @V_VRESTOQDIAM1   DOUBLE PRECISION;
   DECLARE @V_VRESTOQDIAM2   DOUBLE PRECISION;
   DECLARE @V_VRESTOQANT     DOUBLE PRECISION;
   DECLARE @V_VRESTOQANTM1   DOUBLE PRECISION;
   DECLARE @V_VRESTOQANTM2   DOUBLE PRECISION;
   DECLARE @V_VRESTOQBRUT    DOUBLE PRECISION;
   DECLARE @V_VRESTOQBRUM1   DOUBLE PRECISION;
   DECLARE @V_VRESTOQBRUM2   DOUBLE PRECISION;
   DECLARE @V_VRBRUTOANT     DOUBLE PRECISION;
   DECLARE @V_VRBRUTOANTM1   DOUBLE PRECISION;
   DECLARE @V_VRBRUTOANTM2   DOUBLE PRECISION;

   DECLARE @V_VRULTCOMPR     DOUBLE PRECISION;
   DECLARE @V_VRULTCOMPRM1   DOUBLE PRECISION;
   DECLARE @V_VRULTCOMPRM2   DOUBLE PRECISION;

   DECLARE @V_VRCUSPRLISTP   DOUBLE PRECISION;
   DECLARE @V_VRCUSPRLISM1   DOUBLE PRECISION;
   DECLARE @V_VRCUSPRLISM2   DOUBLE PRECISION;
   DECLARE @V_VRCUSLIPRIMP   DOUBLE PRECISION;
   DECLARE @V_VRPRCUSTOAF    DOUBLE PRECISION;
   DECLARE @V_VRPRCUSAFM1    DOUBLE PRECISION;
   DECLARE @V_VRPRCUSAFM2    DOUBLE PRECISION;
   DECLARE @V_IDUTILDTPROC   VARCHAR(01);

   DECLARE @V_VRCLOGDIA      DOUBLE PRECISION;
   DECLARE @V_VRCLOGDIAANT   DOUBLE PRECISION;
   DECLARE @V_VRCLOGDIAM1    DOUBLE PRECISION;
   DECLARE @V_VRCLOGDIAAM1   DOUBLE PRECISION;
   DECLARE @V_VRCLOGDIAM2    DOUBLE PRECISION;
   DECLARE @V_VRCLOGDIAAM2   DOUBLE PRECISION;
   DECLARE @V_VRSEGUDIA      DOUBLE PRECISION;
   DECLARE @V_VRSEGUDIAANT   DOUBLE PRECISION;
   DECLARE @V_VRSEGUDIAM1    DOUBLE PRECISION;
   DECLARE @V_VRSEGUDIAAM1   DOUBLE PRECISION;
   DECLARE @V_VRSEGUDIAM2    DOUBLE PRECISION;
   DECLARE @V_VRSEGUDIAAM2   DOUBLE PRECISION;
   DECLARE @V_CDPRODUTO      VARCHAR(10);
   DECLARE @V_CDPRODACOR     VARCHAR(10);
   DECLARE @V_CDPRODLIST     VARCHAR(10);
   DECLARE @V_FORNECEDOR     VARCHAR(21);
   DECLARE @V_CDFORNECACOR   VARCHAR(21);
   DECLARE @V_CDFORNECLIST   VARCHAR(21);
   DECLARE @V_CDFORNEOL      VARCHAR(21);
   DECLARE @V_VRPESOBRUTPR   DOUBLE PRECISION;
   DECLARE @V_VRPECUSTOLOG   DOUBLE PRECISION;
   DECLARE @V_VRVRCUSTOLOG   DOUBLE PRECISION;
   DECLARE @V_VRPESEGURLOG   DOUBLE PRECISION;
   DECLARE @V_VRCUSTOLOG     DOUBLE PRECISION;
   DECLARE @V_VRCUSTOSEG     DOUBLE PRECISION;
   DECLARE @V_VRLIQCMPPROD   DOUBLE PRECISION;
   DECLARE @V_VRBRUCMPPROD   DOUBLE PRECISION;
   DECLARE @V_VRLIQCMPPRM1   DOUBLE PRECISION;
   DECLARE @V_VRLIQCMPPRM2   DOUBLE PRECISION;
   DECLARE @V_VRCUSTLOGCMP   DOUBLE PRECISION;
   DECLARE @V_VRCUSTLCMPM1   DOUBLE PRECISION;
   DECLARE @V_VRCUSTLCMPM2   DOUBLE PRECISION;
   DECLARE @V_VRSEGULOGCMP   DOUBLE PRECISION;
   DECLARE @V_VRSEGULCMPM1   DOUBLE PRECISION;
   DECLARE @V_VRSEGULCMPM2   DOUBLE PRECISION;
   DECLARE @V_CDFORNCUSTAF   VARCHAR(21);
   DECLARE @V_CDPRODCUSTAF   VARCHAR(10);
   DECLARE @V_CDFILIAL       VARCHAR(04);
   DECLARE @V_VRFATOCONV     DOUBLE PRECISION;
   DECLARE @V_COUNT          DOUBLE PRECISION;
   DECLARE @V_DTMAXESTOQ     DATETIME; 


   SET @V_DTREFERENCIA  = NULL;
   SET @V_QTESTOQDIA    = 0;
   SET @V_QTESTOQANT    = 0;
   SET @V_VRESTOQDIA    = 0;
   SET @V_VRESTOQDIAM1  = 0;
   SET @V_VRESTOQDIAM2  = 0;
   SET @V_VRESTOQANT    = 0;
   SET @V_VRESTOQANTM1  = 0;
   SET @V_VRESTOQANTM2  = 0;
   SET @V_VRESTOQBRUT   = 0;
   SET @V_VRESTOQBRUM1  = 0;
   SET @V_VRESTOQBRUM2  = 0;
   SET @V_VRBRUTOANT    = 0;
   SET @V_VRBRUTOANTM1  = 0;
   SET @V_VRBRUTOANTM2  = 0;
   SET @V_VRULTCOMPR    = 0;
   SET @V_VRULTCOMPRM1  = 0;
   SET @V_VRULTCOMPRM2  = 0;
   SET @V_VRCUSPRLISTP  = 0;
   SET @V_VRCUSPRLISM1  = 0;
   SET @V_VRCUSPRLISM2  = 0;
   SET @V_VRCUSLIPRIMP  = 0;
   SET @V_VRPRCUSTOAF   = 0;
   SET @V_VRPRCUSAFM1   = 0;
   SET @V_VRPRCUSAFM2   = 0;
   SET @V_VRCUSTO       = 0;


   SET @V_VRCLOGDIA     = 0;
   SET @V_VRCLOGDIAANT  = 0;
   SET @V_VRCLOGDIAM1   = 0;
   SET @V_VRCLOGDIAAM1  = 0;
   SET @V_VRCLOGDIAM2   = 0;
   SET @V_VRCLOGDIAAM2  = 0;
   SET @V_VRSEGUDIA     = 0;
   SET @V_VRSEGUDIAANT  = 0;
   SET @V_VRSEGUDIAM1   = 0;
   SET @V_VRSEGUDIAAM1  = 0;
   SET @V_VRSEGUDIAM2   = 0;
   SET @V_VRSEGUDIAAM2  = 0;
   SET @V_VRPESOBRUTPR  = 0;
   SET @V_VRPECUSTOLOG  = 0;
   SET @V_VRVRCUSTOLOG  = 0;
   SET @V_VRPESEGURLOG  = 0;
   SET @V_VRCUSTOLOG    = 0;
   SET @V_VRCUSTOSEG    = 0;
   SET @V_VRLIQCMPPROD  = 0;
   SET @V_VRBRUCMPPROD  = 0;
   SET @V_VRLIQCMPPRM1  = 0;
   SET @V_VRLIQCMPPRM2  = 0;
   SET @V_VRCUSTLOGCMP  = 0;
   SET @V_VRCUSTLCMPM1  = 0;
   SET @V_VRCUSTLCMPM2  = 0;
   SET @V_VRSEGULOGCMP  = 0;
   SET @V_VRSEGULCMPM1  = 0;
   SET @V_VRSEGULCMPM2  = 0;
   SET @V_COUNT         = 0;

   IF @P_NRTIPOCUSTO IN (1,2,3,4,20,21,22,23,24,25,26,27,40,41,42,43,49,50,53,54) BEGIN /*Custos que tem valores na tabela POSESTCONS*/
      IF (@P_NRTIPOCUSTO IN (1,2,20,21,22,23,40,41,49,50,51,52)) BEGIN         /*Custo Mdio da poca*/
         SELECT @V_IDUTILDTPROC = MAX(ISNULL(IDUTILDTPROC,'N'))
         FROM PARAMGERAL;
         IF @V_IDUTILDTPROC = 'S' BEGIN
            SELECT @V_DTREFERENCIA = ISNULL(MAX(DTPROCESSA), @P_DTATUAL)
            FROM PARAMFILIAL P
            WHERE CDFILIAL = @P_CDFILIAL;
         END
         ELSE BEGIN
            SET @V_DTREFERENCIA = @P_DTATUAL;
         END;
      END
      ELSE BEGIN
         SET @V_DTREFERENCIA = @P_DTREFERENCIA;
      END;

      SELECT @V_DTMAXESTOQ = DTPOSIESTQ
      FROM POSESTDIA C
      WHERE C.CDFILIAL       = @P_CDFILIAL   AND
            C.CDPRODUTO      = @P_CDPRODUTO  AND
            C.CDALMOXARIFE   = @P_ALMOXARIFE AND
            C.CDLOCALESTOQ   = @P_LOCALESTOQ AND
            C.NRLOTEESTQ     = @P_NRLOTEESTQ AND
            C.NRSUBLOTE      = @P_NRSUBLOTE  AND
            C.DTPOSIESTQ     <= (@V_DTREFERENCIA+1);

      SELECT @V_QTESTOQDIA   = ISNULL(MAX(QTESTOQDIA),0),
             @V_QTESTOQANT   = ISNULL(MAX(QTESTOQANT),0),
             @V_VRESTOQDIA   = ISNULL(MAX(VRESTOQDIA),0),
             @V_VRESTOQDIAM1 = ISNULL(MAX(VRESTOQDIAM1),0),
             @V_VRESTOQDIAM2 = ISNULL(MAX(VRESTOQDIAM2),0),
             @V_VRESTOQANT   = ISNULL(MAX(VRESTOQANT),0),
             @V_VRESTOQANTM1 = ISNULL(MAX(VRESTOQANTM1),0),
             @V_VRESTOQANTM2 = ISNULL(MAX(VRESTOQANTM2),0),
             @V_VRESTOQBRUT  = ISNULL(MAX(VRESTOQBRUT),0),
             @V_VRESTOQBRUM1 = ISNULL(MAX(VRESTOQBRUM1),0),
             @V_VRESTOQBRUM2 = ISNULL(MAX(VRESTOQBRUM2),0),
             @V_VRBRUTOANT   = ISNULL(MAX(VRBRUTOANT),0),
             @V_VRBRUTOANTM1 = ISNULL(MAX(VRBRUTOANTM1),0),
             @V_VRBRUTOANTM2 = ISNULL(MAX(VRBRUTOANTM2),0),
             @V_VRCLOGDIA    = ISNULL(MAX(VRCLOGDIA),0),
             @V_VRCLOGDIAANT = ISNULL(MAX(VRCLOGDIAANT),0),
             @V_VRCLOGDIAM1  = ISNULL(MAX(VRCLOGDIAM1),0),
             @V_VRCLOGDIAAM1 = ISNULL(MAX(VRCLOGDIAAM1),0),
             @V_VRCLOGDIAM2  = ISNULL(MAX(VRCLOGDIAM2),0),
             @V_VRCLOGDIAAM2 = ISNULL(MAX(VRCLOGDIAAM2),0),
             @V_VRSEGUDIA    = ISNULL(MAX(VRSEGUDIA),0),
             @V_VRSEGUDIAANT = ISNULL(MAX(VRSEGUDIAANT),0),
             @V_VRSEGUDIAM1  = ISNULL(MAX(VRSEGUDIAM1),0),
             @V_VRSEGUDIAAM1 = ISNULL(MAX(VRSEGUDIAAM1),0),
             @V_VRSEGUDIAM2  = ISNULL(MAX(VRSEGUDIAM2),0),
             @V_VRSEGUDIAAM2 = ISNULL(MAX(VRSEGUDIAAM2),0)
      FROM POSESTDIA C
      WHERE C.CDFILIAL       = @P_CDFILIAL   AND
            C.CDPRODUTO      = @P_CDPRODUTO  AND
            C.CDALMOXARIFE   = @P_ALMOXARIFE AND
            C.CDLOCALESTOQ   = @P_LOCALESTOQ AND
            C.NRLOTEESTQ     = @P_NRLOTEESTQ AND
            C.NRSUBLOTE      = @P_NRSUBLOTE  AND
            C.DTPOSIESTQ     = @V_DTMAXESTOQ;

      IF (@V_QTESTOQDIA <> 0) BEGIN
         SET @V_VRESTOQDIA   = (@V_VRESTOQDIA/@V_QTESTOQDIA);
         SET @V_VRESTOQDIAM1 = (@V_VRESTOQDIAM1/@V_QTESTOQDIA);
         SET @V_VRESTOQDIAM2 = (@V_VRESTOQDIAM2/@V_QTESTOQDIA);
         SET @V_VRESTOQBRUT  = (@V_VRESTOQBRUT/@V_QTESTOQDIA);
         SET @V_VRESTOQBRUM1 = (@V_VRESTOQBRUM1/@V_QTESTOQDIA);
         SET @V_VRESTOQBRUM2 = (@V_VRESTOQBRUM2/@V_QTESTOQDIA);

         SET @V_VRCLOGDIA    = (@V_VRCLOGDIA/@V_QTESTOQDIA);
         SET @V_VRCLOGDIAM1  = (@V_VRCLOGDIAM1/@V_QTESTOQDIA);
         SET @V_VRCLOGDIAM2  = (@V_VRCLOGDIAM2/@V_QTESTOQDIA);
         SET @V_VRSEGUDIA    = (@V_VRSEGUDIA/@V_QTESTOQDIA);
         SET @V_VRSEGUDIAM1  = (@V_VRSEGUDIAM1/@V_QTESTOQDIA);
         SET @V_VRSEGUDIAM2  = (@V_VRSEGUDIAM2/@V_QTESTOQDIA);

      END
      ELSE BEGIN
         IF (@V_QTESTOQANT <> 0) BEGIN
            SET @V_VRESTOQDIA   = (@V_VRESTOQANT/@V_QTESTOQANT);
            SET @V_VRESTOQDIAM1 = (@V_VRESTOQANTM1/@V_QTESTOQANT);
            SET @V_VRESTOQDIAM2 = (@V_VRESTOQANTM2/@V_QTESTOQANT);
            SET @V_VRESTOQBRUT  = (@V_VRBRUTOANT/@V_QTESTOQANT);
            SET @V_VRESTOQBRUM1 = (@V_VRBRUTOANTM1/@V_QTESTOQANT);
            SET @V_VRESTOQBRUM2 = (@V_VRBRUTOANTM2/@V_QTESTOQANT);

            SET @V_VRCLOGDIA     = (@V_VRCLOGDIAANT/@V_QTESTOQANT);
            SET @V_VRCLOGDIAM1   = (@V_VRCLOGDIAAM1/@V_QTESTOQANT);
            SET @V_VRCLOGDIAM2   = (@V_VRCLOGDIAAM2/@V_QTESTOQANT);
            SET @V_VRSEGUDIA     = (@V_VRSEGUDIAANT/@V_QTESTOQANT);
            SET @V_VRSEGUDIAM1   = (@V_VRSEGUDIAAM1/@V_QTESTOQANT);
            SET @V_VRSEGUDIAM2   = (@V_VRSEGUDIAAM2/@V_QTESTOQANT);
         END
         ELSE BEGIN
            SET @V_VRESTOQDIA   = 0;
            SET @V_VRESTOQDIAM1 = 0;
            SET @V_VRESTOQDIAM2 = 0;
            SET @V_VRESTOQBRUT  = 0;
            SET @V_VRESTOQBRUM1 = 0;
            SET @V_VRESTOQBRUM2 = 0;

            SET @V_VRCLOGDIA    = 0;
            SET @V_VRCLOGDIAM1  = 0;
            SET @V_VRCLOGDIAM2  = 0;
            SET @V_VRSEGUDIA    = 0;
            SET @V_VRSEGUDIAM1  = 0;
            SET @V_VRSEGUDIAM2  = 0;
         END;
      END;

      IF @P_NRTIPOCUSTO IN (1,3,40,42) BEGIN      /*Custo Mdio Lquido*/
         SET @V_VRCUSTO = @V_VRESTOQDIA;
         IF @P_NRTIPOCUSTO IN (40,42) BEGIN /*Custo Mdio Lquido(Logistico)*/
            SET @V_VRCUSTO = @V_VRCUSTO + @V_VRCLOGDIA + @V_VRSEGUDIA;
         END;
      END
      ELSE BEGIN
        IF @P_NRTIPOCUSTO IN (2,4,41,43) BEGIN   /*Custo Mdio Bruto*/
           SET @V_VRCUSTO = @V_VRESTOQBRUT;
           IF @P_NRTIPOCUSTO IN (41,43) BEGIN  /*Custo Mdio Bruto(Logistico)*/
              SET @V_VRCUSTO = @V_VRCUSTO + @V_VRCLOGDIA + @V_VRSEGUDIA;
           END;
        END
        ELSE BEGIN
          IF @P_NRTIPOCUSTO IN (20,24,49,53) BEGIN /*Custo Mdio Lquido (ndice 01)*/
             SET @V_VRCUSTO = @V_VRESTOQDIAM1;
             IF @P_NRTIPOCUSTO IN (49,53) BEGIN  /*Custo Mdio Lquido(Logistico) Indice 01*/
                SET @V_VRCUSTO = @V_VRCUSTO + @V_VRCLOGDIAM1+@V_VRSEGUDIAM1;
             END;

          END
          ELSE BEGIN
            IF @P_NRTIPOCUSTO IN (21,25,50,54) BEGIN /*Custo Mdio Lquido (ndice 02)*/
               SET @V_VRCUSTO = @V_VRESTOQDIAM2;
               IF @P_NRTIPOCUSTO IN (40,54) BEGIN  /*Custo Mdio Lquido(Logistico) Indice 02*/
                  SET @V_VRCUSTO = @V_VRCUSTO + @V_VRCLOGDIAM2+@V_VRSEGUDIAM2;
               END;
            END
            ELSE BEGIN
              IF @P_NRTIPOCUSTO IN (22,26,51,55) BEGIN /*Custo Mdio Bruto (ndice 01)*/
                 SET @V_VRCUSTO = @V_VRESTOQBRUM1;
                 IF @P_NRTIPOCUSTO IN (51,55) BEGIN  /*Custo Mdio Bruto(Logistico) Indice 01*/
                    SET @V_VRCUSTO = @V_VRCUSTO + @V_VRCLOGDIAM1+@V_VRSEGUDIAM1;
                 END;
              END
              ELSE BEGIN
                IF @P_NRTIPOCUSTO IN (23,27,52,56) BEGIN /*Custo Mdio Bruto (ndice 02)*/
                   SET @V_VRCUSTO = @V_VRESTOQBRUM2;
                   IF @P_NRTIPOCUSTO IN (52,56) BEGIN  /*Custo Mdio Bruto(Logistico) Indice 02*/
                      SET @V_VRCUSTO = @V_VRCUSTO + @V_VRCLOGDIAM2+@V_VRSEGUDIAM2;
                   END;

                END;
              END;
            END;
          END;
        END;
      END;
   END
   ELSE BEGIN
      IF @P_NRTIPOCUSTO IN (6,7,10,30,31,32,33,44,48,45,59,60,61,62) BEGIN /*Custos que tem valores na tabela CUSTOSUP*/
         SET @V_CDFORNEOL = NULL;
         IF @P_NRTIPOCUSTO IN (44, 45, 48, 59, 60,61,62) BEGIN
            SELECT
               @V_CDFILIAL = MAX(P.CDFILIOPELOG)
            FROM
               PARAMPRODSUP P
            WHERE
               P.CDFILIAL  = @P_CDFILIAL AND
               P.CDPRODUTO = @P_CDPRODUTO;
            IF @V_CDFILIAL IS NULL BEGIN
               SET @V_CDFILIAL = @P_CDFILIAL;
            END
            ELSE BEGIN
               SELECT
                  @V_CDFORNEOL = MAX(FILIAL.CDFORNOPELOG)
               FROM
                  FILIAL
               WHERE
                  FILIAL.CDFILIAL = @V_CDFILIAL;
            END;
         END
         ELSE BEGIN
            SET @V_CDFILIAL = @P_CDFILIAL;
         END;


         IF @P_NRTIPOCUSTO IN (6, 10, 30, 31, 44, 48, 61, 62) BEGIN
            SELECT
               @V_VRCUSPRLISTP = ISNULL(MAX(VRCUSPRLISTP),0),
               @V_VRCUSPRLISM1 = ISNULL(MAX(VRCUSPRLISM1),0),
               @V_VRCUSPRLISM2 = ISNULL(MAX(VRCUSPRLISM2),0),
               @V_VRCUSLIPRIMP = ISNULL(MAX(VRCUSLIPRIMP),0),
               @V_CDPRODACOR   = MAX(CDPRODACOR),
               @V_CDPRODLIST   = MAX(CDPRODLIST),
               @V_CDFORNECLIST = MAX(CDFORNECLIST),
               @V_CDFORNECACOR = MAX(CDFORNECACOR)
            FROM CUSTOSUP
            WHERE CDFILIAL   = @V_CDFILIAL  AND
                  CDPRODUTO  = @P_CDPRODUTO AND
                  DTCUSTOSUP = (SELECT MAX(DTCUSTOSUP)
                                FROM CUSTOSUP
                                WHERE CDFILIAL  = @V_CDFILIAL  AND
                                      CDPRODUTO = @P_CDPRODUTO AND
                                      VRCUSPRLISTP IS NOT NULL AND
                                      DTCUSTOSUP <= @P_DTREFERENCIA);
         END;

         IF @P_NRTIPOCUSTO IN (7, 32, 33, 45, 59, 60) BEGIN
            SELECT
               @V_VRPRCUSTOAF  = ISNULL(MAX(VRPRCUSTOAF),0),
               @V_VRPRCUSAFM1  = ISNULL(MAX(VRPRCUSAFM1),0),
               @V_VRPRCUSAFM2  = ISNULL(MAX(VRPRCUSAFM2),0),
               @V_CDFORNCUSTAF = MAX(CDFORNCUSTAF),
               @V_CDPRODCUSTAF = MAX(CDPRODCUSTAF)
            FROM CUSTOSUP
            WHERE CDFILIAL   = @V_CDFILIAL  AND
                  CDPRODUTO  = @P_CDPRODUTO AND
                  DTCUSTOSUP = (SELECT MAX(DTCUSTOSUP)
                                FROM CUSTOSUP
                                WHERE CDFILIAL  = @V_CDFILIAL  AND
                                      CDPRODUTO = @P_CDPRODUTO AND
                                      VRPRCUSTOAF IS NOT NULL AND
                                      DTCUSTOSUP <= @P_DTREFERENCIA);
         END;

/*      Lista de preo e cotao(Logistico)    [06,10][44,48]*/

         IF (@V_CDPRODACOR IS NOT NULL) BEGIN
            SET @V_CDPRODUTO   = @V_CDPRODACOR;
            SET @V_FORNECEDOR  = @V_CDFORNECACOR;
         END;

         IF (@V_CDPRODLIST IS NOT NULL) BEGIN
            SET @V_CDPRODUTO    = @V_CDPRODLIST;
            SET @V_FORNECEDOR   = @V_CDFORNECLIST;
         END;

         IF (@V_CDPRODCUSTAF IS NOT NULL) BEGIN
            SET @V_CDPRODUTO    = @V_CDPRODCUSTAF;
            SET @V_FORNECEDOR   = @V_CDFORNCUSTAF;
         END;

         IF @P_NRTIPOCUSTO IN (44, 45, 48, 59, 60,61,62) BEGIN
            SELECT
               @V_VRFATOCONV = ISNULL(MAX(VRFATOCONV), 1)
            FROM
               PRODUTO
            WHERE
               PRODUTO.CDPRODUTO = @V_CDPRODUTO;

            SELECT
               @V_VRPESOBRUTPR = X.VRPESOBRUTPR,
               @V_VRPECUSTOLOG = C.VRPECUSTOLOG,
               @V_VRVRCUSTOLOG = C.VRVRCUSTOLOG,
               @V_VRPESEGURLOG = C.VRPESEGURLOG
            FROM
               PRODUTO P,
               CUSTOLOGIST C,
               (SELECT
                   ISNULL(MAX(A.VRPESOBRUTPR),0) VRPESOBRUTPR,
                      MAX(P.CDARVPROD) CDPRODUTO,
                      MAX(C.DTINIVIGCLOG) DTINIVIGCLOG
                FROM
                   PRODUTO P,
                   CUSTOLOGIST C,
                   PRODUTO A
                WHERE
                   (CHARINDEX(P.CDARVPROD, A.CDARVPROD) = 1)       AND
                   A.CDPRODUTO    = @V_CDPRODUTO                   AND
                   P.CDPRODUTO    = C.CDPRODUTO                    AND
                   C.CDFILIAL     = @P_CDFILIAL                    AND
                   C.CDFORNECED   = ISNULL(@V_CDFORNEOL, @V_FORNECEDOR) AND
                   C.DTINIVIGCLOG = (SELECT MAX(DTINIVIGCLOG)
                                     FROM CUSTOLOGIST A, PRODUTO B, PRODUTO Q
                                     WHERE
                                        A.CDFILIAL     = C.CDFILIAL                AND
                                        A.CDFORNECED   = C.CDFORNECED              AND
                                        (CHARINDEX(Q.CDARVPROD, B.CDARVPROD) = 1)  AND
                                        B.CDPRODUTO    = @V_CDPRODUTO              AND
                                        Q.CDPRODUTO    = A.CDPRODUTO               AND
                                        A.DTINIVIGCLOG <= @P_DTREFERENCIA))X
            WHERE C.CDFILIAL     = @P_CDFILIAL                         AND
                  C.CDPRODUTO    = P.CDPRODUTO                         AND
                  C.CDPRODUTO    = X.CDPRODUTO                         AND
                  C.CDFORNECED   = ISNULL(@V_CDFORNEOL, @V_FORNECEDOR) AND
                  C.DTINIVIGCLOG = X.DTINIVIGCLOG;
         END;

         IF @V_VRFATOCONV IS NULL BEGIN
            SET @V_VRFATOCONV = 1;
         END;

         IF @P_NRTIPOCUSTO IN (6, 44) BEGIN           /*Custo de Cotao (Lista de Preos)*/
            SET @V_VRCUSTO = @V_VRCUSPRLISTP;
            IF ISNULL(@V_VRCUSTO, 0) > 0 BEGIN
               IF @P_NRTIPOCUSTO = 44  BEGIN
                  IF (@V_VRVRCUSTOLOG > 0) AND (@V_VRPESOBRUTPR > 0) BEGIN
                     SET @V_VRCUSTOLOG = (@V_VRPESOBRUTPR * @V_VRVRCUSTOLOG)/@V_VRFATOCONV;
                  END;

                  IF (@V_VRPECUSTOLOG > 0) BEGIN
                     SET @V_VRCUSTOLOG = (@V_VRCUSTO * @V_VRPECUSTOLOG)/100;
                  END;

                  IF (@V_VRPESEGURLOG > 0) BEGIN
                     SET @V_VRCUSTOSEG = (@V_VRCUSTO * @V_VRPESEGURLOG)/100;
                  END;

                  SET @V_VRCUSTO = @V_VRCUSTO + ISNULL(@V_VRCUSTOLOG, 0) + ISNULL(@V_VRCUSTOSEG, 0);
               END;
            END;
         END
         ELSE BEGIN
            IF @P_NRTIPOCUSTO IN (7,45) BEGIN        /*Custo da ltima compra(AF)*/
               SET @V_VRCUSTO = @V_VRPRCUSTOAF;
               IF ISNULL(@V_VRCUSTO, 0) > 0 BEGIN
                  IF @P_NRTIPOCUSTO = 45  BEGIN
                     IF (@V_VRVRCUSTOLOG > 0) AND (@V_VRPESOBRUTPR > 0) BEGIN
                        SET @V_VRCUSTOLOG = (@V_VRPESOBRUTPR * @V_VRVRCUSTOLOG)/@V_VRFATOCONV;
                     END;

                     IF (@V_VRPECUSTOLOG > 0) BEGIN
                        SET @V_VRCUSTOLOG = (@V_VRCUSTO * @V_VRPECUSTOLOG)/100;
                     END;

                     IF (@V_VRPESEGURLOG > 0) BEGIN
                        SET @V_VRCUSTOSEG = (@V_VRCUSTO * @V_VRPESEGURLOG)/100;
                     END;

                     SET @V_VRCUSTO = @V_VRCUSTO + ISNULL(@V_VRCUSTOLOG, 0) + ISNULL(@V_VRCUSTOSEG, 0);
                  END;
               END;
            END
            ELSE BEGIN
              IF @P_NRTIPOCUSTO IN (10,48) BEGIN       /*Custo de Cotao(Lista de Preos com Imposto)*/
                 SET @V_VRCUSTO     = @V_VRCUSLIPRIMP;
                 SET @V_SEM_IMPOSTO = @V_VRCUSPRLISTP;
                 IF ISNULL(@V_VRCUSTO, 0) > 0 BEGIN
                    IF @P_NRTIPOCUSTO = 48  BEGIN
                       IF (@V_VRVRCUSTOLOG > 0) AND (@V_VRPESOBRUTPR > 0) BEGIN
                          SET @V_VRCUSTOLOG = (@V_VRPESOBRUTPR * @V_VRVRCUSTOLOG)/@V_VRFATOCONV;
                       END;

                       IF (@V_VRPECUSTOLOG > 0) BEGIN
                          SET @V_VRCUSTOLOG = (@V_VRCUSTO * @V_VRPECUSTOLOG)/100;
                       END;

                       IF (@V_VRPESEGURLOG > 0) BEGIN
                          SET @V_VRCUSTOSEG = (@V_SEM_IMPOSTO * @V_VRPESEGURLOG)/100;
                       END;

                       SET @V_VRCUSTO = @V_VRCUSTO + ISNULL(@V_VRCUSTOLOG, 0) + ISNULL(@V_VRCUSTOSEG, 0);
                    END;
                 END;
              END
              ELSE BEGIN
                IF @P_NRTIPOCUSTO IN (30,61) BEGIN       /*Custo de Cotao (Lista de Preos) (ndice 01)*/
                   SET @V_VRCUSTO = @V_VRCUSPRLISM1;
                   IF ISNULL(@V_VRCUSTO, 0) > 0 BEGIN
                      IF @P_NRTIPOCUSTO = 61  BEGIN
                         IF (@V_VRVRCUSTOLOG > 0) AND (@V_VRPESOBRUTPR > 0) BEGIN
                            SET @V_VRCUSTOLOG = (@V_VRPESOBRUTPR * @V_VRVRCUSTOLOG)/@V_VRFATOCONV;
                         END;

                         IF (@V_VRPECUSTOLOG > 0) BEGIN
                            SET @V_VRCUSTOLOG = (@V_VRCUSTO * @V_VRPECUSTOLOG)/100;
                         END;

                         IF (@V_VRPESEGURLOG > 0) BEGIN
                            SET @V_VRCUSTOSEG = (@V_VRCUSTO * @V_VRPESEGURLOG)/100;
                         END;

                         SET @V_VRCUSTO = @V_VRCUSTO + ISNULL(@V_VRCUSTOLOG, 0) + ISNULL(@V_VRCUSTOSEG, 0);
                      END;
                   END;
                END
                ELSE BEGIN
                  IF @P_NRTIPOCUSTO IN (31,62) BEGIN       /*Custo de Cotao (Lista de Preos) (ndice 02)*/
                     SET @V_VRCUSTO = @V_VRCUSPRLISM2;
                     IF ISNULL(@V_VRCUSTO, 0) > 0 BEGIN
                        IF @P_NRTIPOCUSTO = 62  BEGIN
                           IF (@V_VRVRCUSTOLOG > 0) AND (@V_VRPESOBRUTPR > 0) BEGIN
                              SET @V_VRCUSTOLOG = (@V_VRPESOBRUTPR * @V_VRVRCUSTOLOG)/@V_VRFATOCONV;
                           END;
                           IF (@V_VRPECUSTOLOG > 0) BEGIN
                              SET @V_VRCUSTOLOG = (@V_VRCUSTO * @V_VRPECUSTOLOG)/100;
                           END;

                           IF (@V_VRPESEGURLOG > 0) BEGIN
                              SET @V_VRCUSTOSEG = (@V_VRCUSTO * @V_VRPESEGURLOG)/100;
                           END;
                           SET @V_VRCUSTO = @V_VRCUSTO + ISNULL(@V_VRCUSTOLOG, 0) + ISNULL(@V_VRCUSTOSEG, 0);
                        END;
                     END;
                  END
                  ELSE BEGIN
                    IF @P_NRTIPOCUSTO IN (32,59) BEGIN       /*Custo da ltima Compra(AF) (ndice 01)*/
                       SET @V_VRCUSTO = @V_VRPRCUSAFM1;
                       IF ISNULL(@V_VRCUSTO, 0) > 0 BEGIN
                          IF @P_NRTIPOCUSTO = 59  BEGIN
                             IF (@V_VRVRCUSTOLOG > 0) AND (@V_VRPESOBRUTPR > 0) BEGIN
                                SET @V_VRCUSTOLOG = (@V_VRPESOBRUTPR * @V_VRVRCUSTOLOG)/@V_VRFATOCONV;
                             END;

                             IF (@V_VRPECUSTOLOG > 0) BEGIN
                                SET @V_VRCUSTOLOG = (@V_VRCUSTO * @V_VRPECUSTOLOG)/100;
                             END;

                             IF (@V_VRPESEGURLOG > 0) BEGIN
                                SET @V_VRCUSTOSEG = (@V_VRCUSTO * @V_VRPESEGURLOG)/100;
                             END;

                             SET @V_VRCUSTO = @V_VRCUSTO + ISNULL(@V_VRCUSTOLOG, 0) + ISNULL(@V_VRCUSTOSEG, 0);
                          END;
                       END;
                    END
                    ELSE BEGIN
                      IF @P_NRTIPOCUSTO IN (33,60) BEGIN       /*Custo da ltima Compra(AF) (ndice 02)*/
                         SET @V_VRCUSTO = @V_VRPRCUSAFM2;
                         IF ISNULL(@V_VRCUSTO, 0) > 0 BEGIN
                            IF @P_NRTIPOCUSTO = 60  BEGIN
                               IF (@V_VRVRCUSTOLOG > 0) AND (@V_VRPESOBRUTPR > 0) BEGIN
                                  SET @V_VRCUSTOLOG = (@V_VRPESOBRUTPR * @V_VRVRCUSTOLOG)/@V_VRFATOCONV;
                               END;

                               IF (@V_VRPECUSTOLOG > 0) BEGIN
                                  SET @V_VRCUSTOLOG = (@V_VRCUSTO * @V_VRPECUSTOLOG)/100;
                               END;

                               IF (@V_VRPESEGURLOG > 0) BEGIN
                                  SET @V_VRCUSTOSEG = (@V_VRCUSTO * @V_VRPESEGURLOG)/100;
                               END;

                               SET @V_VRCUSTO = @V_VRCUSTO + ISNULL(@V_VRCUSTOLOG, 0) + ISNULL(@V_VRCUSTOSEG, 0);
                            END;
                         END;
                      END;
                    END;
                  END;
                END;
              END;
            END;
         END;
      END
      ELSE BEGIN
         IF @P_NRTIPOCUSTO IN (8,9,28,29,46,47) BEGIN /*Custos que tem valores na tabela POSESTDIA*/
            SELECT @V_VRLIQCMPPROD = MAX(ISNULL(VRLIQCMPPROD,0)),
                   @V_VRBRUCMPPROD = MAX(ISNULL(VRBRUCMPPROD,0)),
                   @V_VRLIQCMPPRM1 = MAX(ISNULL(VRLIQCMPPRM1,0)),
                   @V_VRLIQCMPPRM2 = MAX(ISNULL(VRLIQCMPPRM2,0)),
                   @V_VRCUSTLOGCMP = MAX(ISNULL(VRCUSTLOGCMP,0)),
                   @V_VRCUSTLCMPM1 = MAX(ISNULL(VRCUSTLCMPM1,0)),
                   @V_VRCUSTLCMPM2 = MAX(ISNULL(VRCUSTLCMPM2,0)),
                   @V_VRSEGULOGCMP = MAX(ISNULL(VRSEGULOGCMP,0)),
                   @V_VRSEGULCMPM1 = MAX(ISNULL(VRSEGULCMPM1,0)),
                   @V_VRSEGULCMPM2 = MAX(ISNULL(VRSEGULCMPM2,0))
            FROM CUSTOCMPPROD
            WHERE CDFILIAL       = @P_CDFILIAL       AND
                  CDALMOXARIFE   = @P_ALMOXARIFE     AND
                  CDLOCALESTOQ   = @P_LOCALESTOQ     AND
                  NRLOTEESTQ     = @P_NRLOTEESTQ     AND
                  NRSUBLOTE      = @P_NRSUBLOTE      AND
                  CDPRODUTO      = @P_CDPRODUTO      AND
                  DTCUSTCMPPRO   =(SELECT MAX(C.DTCUSTCMPPRO)
                                   FROM CUSTOCMPPROD C
                                   WHERE C.CDFILIAL       = @P_CDFILIAL   AND
                                         C.CDPRODUTO      = @P_CDPRODUTO  AND
                                         C.CDALMOXARIFE   = @P_ALMOXARIFE AND
                                         C.CDLOCALESTOQ   = @P_LOCALESTOQ AND
                                         C.NRLOTEESTQ     = @P_NRLOTEESTQ AND
                                         C.NRSUBLOTE      = @P_NRSUBLOTE  AND
                                         C.DTCUSTCMPPRO  <= @P_DTREFERENCIA);
            IF @P_NRTIPOCUSTO IN (8,46) BEGIN
               SET @V_VRCUSTO = @V_VRLIQCMPPROD;
               IF @P_NRTIPOCUSTO = 46 BEGIN
                  SET @V_VRCUSTO = @V_VRCUSTO + @V_VRCUSTLOGCMP + @V_VRSEGULOGCMP;
               END;
            END
            ELSE BEGIN
               IF @P_NRTIPOCUSTO IN (9,47) BEGIN
                  SET @V_VRCUSTO = @V_VRBRUCMPPROD;

                  IF @P_NRTIPOCUSTO = 47 BEGIN
                     SET @V_VRCUSTO = @V_VRCUSTO + @V_VRCUSTLOGCMP + @V_VRSEGULOGCMP;
                  END;
               END
               ELSE BEGIN
                  IF @P_NRTIPOCUSTO IN (28,50) BEGIN
                     SET @V_VRCUSTO = @V_VRLIQCMPPRM1;
                     IF @P_NRTIPOCUSTO = 50 BEGIN
                        SET @V_VRCUSTO = @V_VRCUSTO + @V_VRCUSTLCMPM1 + @V_VRSEGULCMPM1;
                     END;
                  END
                  ELSE BEGIN
                     IF @P_NRTIPOCUSTO IN (29,51) BEGIN
                        SET @V_VRCUSTO = @V_VRLIQCMPPRM2;

                        IF @P_NRTIPOCUSTO = 51 BEGIN
                           SET @V_VRCUSTO = @V_VRCUSTO + @V_VRCUSTLCMPM2 + @V_VRSEGULCMPM2;
                        END;
                     END;
                  END;
               END;
            END;
         END
         ELSE BEGIN
            IF @P_NRTIPOCUSTO = 63 BEGIN /*Custos que tem valores na tabela CUSTMEDCTBL*/
               SELECT @V_VRCUSTO = ISNULL(MAX(C.VRCUSTOCTBL),0)
                 FROM CUSTMEDCTBL C, DATA D
                WHERE C.CDFILIAL = @P_CDFILIAL
                  AND C.CDPRODUTO = @P_CDPRODUTO
                  AND D.DTDATA = @P_DTREFERENCIA
                  AND C.DTCOMPCTBL = D.DTMES;
            END;
         END;
      END;
   END;
   RETURN(@V_VRCUSTO);
END
GO


IF EXISTS (SELECT *
           FROM SYSOBJECTS O, SYSUSERS U
           WHERE O.NAME = 'CUSTO_PROD' AND
                 TYPE = 'FN' AND
                 U.UID = O.UID  AND
                 U.NAME = USER)
DROP FUNCTION CUSTO_PROD
GO

CREATE FUNCTION CUSTO_PROD
   (@P_CDFILIAL     VARCHAR(04),
    @P_CDPRODUTO    VARCHAR(10),
    @P_NRTIPOCUST   NUMERIC,
    @P_DTREFERENCIA DATETIME,
    @P_ALMOXARIFE   VARCHAR(06) = ' ',
    @P_LOCALESTOQ   VARCHAR(06) = ' ',
    @P_NRLOTEESTQ   VARCHAR(20) = ' ',
    @P_NRSUBLOTE    VARCHAR(04) = ' ')

RETURNS NUMERIC

AS
BEGIN
   DECLARE @V_CUSTO NUMERIC;

   SELECT @V_CUSTO = TEKNISA.CUSTO_PRODUTO_LCEST(@P_CDFILIAL,
                                                 @P_CDPRODUTO,
                                                 @P_NRTIPOCUST,
                                                 @P_DTREFERENCIA,
                                                 @P_DTREFERENCIA,
                                                 @P_ALMOXARIFE,
                                                 @P_LOCALESTOQ,
                                                 @P_NRLOTEESTQ,
                                                 @P_NRSUBLOTE)
   FROM PARAMGERAL;

   RETURN(@V_CUSTO);
END
GO

-- VERSAO --
EXEC ATUVEROBJ 'CUSTO_PROD_LCEST','5.01.007';
GO

